# -*- coding: UTF-8 -*-

# Config
from Components.config import config

# GUI (System)
from enigma import getDesktop, eEPGCache

# GUI (Screens)
from Screens.Screen import Screen
from Screens.ChoiceBox import ChoiceBox

# GUI (Components)
from Components.ActionMap import ActionMap
from Components.Sources.List import List
from Components.Sources.StaticText import StaticText
from Components.Sources.Event import Event

from ServiceReference import ServiceReference
from Tools.FuzzyDate import FuzzyTime

sz_w = getDesktop(0).size().width()

class AutoTimerPreview(Screen):
	"""Preview Timers which would be set"""

	if sz_w == 1920:
		skinNormal = """
		<screen name="AutoTimerPreview" position="center,170" size="1200,820" title="Preview AutoTimer">
		<ePixmap pixmap="Default-FHD/skin_default/buttons/red.svg" position="10,5" scale="stretch" size="350,70" />
		<ePixmap pixmap="Default-FHD/skin_default/buttons/yellow.svg" position="360,5" scale="stretch" size="350,70" />
		<widget backgroundColor="#9f1313" font="Regular;30" halign="center" position="10,5" render="Label" foregroundColor="white" shadowColor="black" shadowOffset="-2,-2" size="350,70" source="key_red" transparent="1" valign="center" zPosition="1" />
		<widget backgroundColor="#a08500" font="Regular;30" halign="center" position="360,5" render="Label" foregroundColor="white" shadowColor="black" shadowOffset="-2,-2" size="350,70" source="key_yellow" transparent="1" valign="center" zPosition="1" />
		<widget font="Regular;34" halign="right" position="1050,25" render="Label" size="120,40" source="global.CurrentTime">
			<convert type="ClockToText">Default</convert>
		</widget>
		<widget font="Regular;34" halign="right" position="800,25" render="Label" size="240,40" source="global.CurrentTime">
			<convert type="ClockToText">Date</convert>
		</widget>
		<eLabel backgroundColor="grey" position="10,80" size="1180,1" />
		<widget backgroundColor="background" enableWrapAround="1" position="10,90" render="Listbox" scrollbarMode="showOnDemand" size="1180,721" source="timerlist" transparent="1">
			<convert type="TemplatedMultiContent">
			{"template":[
			MultiContentEntryText(pos=(10,0),size=(1160,38),text=0,font=0,flags=RT_HALIGN_LEFT|RT_VALIGN_CENTER),
			MultiContentEntryText(pos=(10,38),size=(1160,32),text=3,font=1,flags=RT_HALIGN_LEFT|RT_VALIGN_CENTER,color=0xa0a0a0),
			MultiContentEntryText(pos=(10,70),size=(730,32),text=4,font=1,flags=RT_HALIGN_LEFT|RT_VALIGN_CENTER,color=0xa0a0a0),
			MultiContentEntryText(pos=(760,70),size=(410,32),text=2,font=1,flags=RT_HALIGN_RIGHT|RT_VALIGN_CENTER,color=0xa0a0a0),
			],
			"fonts":[gFont("Regular",32),gFont("Regular",28) ],"itemHeight":103}
			</convert>
		</widget>
		</screen>"""
		skinBig = """
		<screen name="AutoTimerPreviewBig" position="center,100" size="1800,960" title="Preview AutoTimer">
		<ePixmap pixmap="Default-FHD/skin_default/buttons/red.svg" position="10,5" scale="stretch" size="350,70" />
		<ePixmap pixmap="Default-FHD/skin_default/buttons/yellow.svg" position="360,5" scale="stretch" size="350,70" />
		<widget backgroundColor="#9f1313" font="Regular;30" halign="center" position="10,5" render="Label" foregroundColor="white" shadowColor="black" shadowOffset="-2,-2" size="350,70" source="key_red" transparent="1" valign="center" zPosition="1" />
		<widget backgroundColor="#a08500" font="Regular;30" halign="center" position="360,5" render="Label" foregroundColor="white" shadowColor="black" shadowOffset="-2,-2" size="350,70" source="key_yellow" transparent="1" valign="center" zPosition="1" />
		<widget font="Regular;34" halign="right" position="1650,25" render="Label" size="120,40" source="global.CurrentTime">
			<convert type="ClockToText">Default</convert>
		</widget>
		<widget font="Regular;34" halign="right" position="1400,25" render="Label" size="240,40" source="global.CurrentTime">
			<convert type="ClockToText">Date</convert>
		</widget>
		<eLabel backgroundColor="grey" position="10,80" size="1780,1" />
		<widget backgroundColor="background" enableWrapAround="1" position="10,90" render="Listbox" scrollbarMode="showOnDemand" size="1780,618" source="timerlist" transparent="1">
			<convert type="TemplatedMultiContent">
			{"template":[
			MultiContentEntryText(pos=(10,0),size=(1760,38),text=0,font=0,flags=RT_HALIGN_LEFT|RT_VALIGN_CENTER),
			MultiContentEntryText(pos=(10,38),size=(1760,32),text=3,font=1,flags=RT_HALIGN_LEFT|RT_VALIGN_CENTER,color=0xa0a0a0),
			MultiContentEntryText(pos=(10,70),size=(1030,32),text=4,font=1,flags=RT_HALIGN_LEFT|RT_VALIGN_CENTER,color=0xa0a0a0),
			MultiContentEntryText(pos=(1060,70),size=(710,32),text=2,font=1,flags=RT_HALIGN_RIGHT|RT_VALIGN_CENTER,color=0xa0a0a0),
			],
			"fonts":[gFont("Regular",32),gFont("Regular",28) ],"itemHeight":103}
			</convert>
		</widget>
		<eLabel backgroundColor="grey" position="10,715" size="1780,1" />
		<widget font="Regular;32" position="10,730" render="Label" size="1780,40" source="Event">
			<convert type="EventName">Name</convert>
		</widget>
		<widget font="Regular;28" position="10,790" render="Label" size="1780,160" source="Event">
			<convert type="EventName">FullDescription</convert>
		</widget>
		</screen>"""
	else:
		skinNormal = """
		<screen name="AutoTimerPreview" title="Preview AutoTimer" position="center,120" size="820,520">
		<ePixmap pixmap="skin_default/buttons/red.png" position="10,5" size="200,40" />
		<ePixmap pixmap="skin_default/buttons/yellow.png" position="210,5" size="200,40" />
		<widget source="key_red" render="Label" position="10,5" size="200,40" zPosition="1" font="Regular;20" halign="center" valign="center" backgroundColor="#9f1313" transparent="1" foregroundColor="white" shadowColor="black" shadowOffset="-2,-2"/>
		<widget source="key_yellow" render="Label" position="210,5" size="200,40" zPosition="1" font="Regular;20" halign="center" valign="center" backgroundColor="#a08500" transparent="1" foregroundColor="white" shadowColor="black" shadowOffset="-2,-2"/>
		<eLabel	position="10,50" size="800,1" backgroundColor="grey"/>
		<widget source="timerlist" render="Listbox" position="10,60" size="800,432" enableWrapAround="1" scrollbarMode="showOnDemand">
			<convert type="TemplatedMultiContent">
			{"template": [
			MultiContentEntryText(pos=(2,2), size=(790,24), text = 3, font = 0, flags = RT_HALIGN_LEFT|RT_VALIGN_CENTER),
			MultiContentEntryText(pos=(2,26), size=(790,30), text = 0, font = 1, flags = RT_HALIGN_LEFT|RT_VALIGN_CENTER),
			MultiContentEntryText(pos=(2,50), size=(300,20), text = 4, font = 1, flags = RT_HALIGN_LEFT|RT_VALIGN_CENTER),
			MultiContentEntryText(pos=(320,50), size=(470,20), text = 2, font = 1, flags = RT_HALIGN_RIGHT|RT_VALIGN_CENTER),
			],
			"fonts": [gFont("Regular",20),gFont("Regular",18)],
			"itemHeight": 72
			}
			</convert>
		</widget>
		</screen>"""
		skinBig = """
		<screen name="AutoTimerPreviewBig" title="Preview AutoTimer" position="center,70" size="1120,625">
		<ePixmap pixmap="skin_default/buttons/red.png" position="10,5" size="200,40" />
		<ePixmap pixmap="skin_default/buttons/yellow.png" position="210,5" size="200,40" />
		<widget source="key_red" render="Label" position="10,5" size="200,40" zPosition="1" font="Regular;20" halign="center" valign="center" backgroundColor="#9f1313" transparent="1" foregroundColor="white" shadowColor="black" shadowOffset="-2,-2"/>
		<widget source="key_yellow" render="Label" position="210,5" size="200,40" zPosition="1" font="Regular;20" halign="center" valign="center" backgroundColor="#a08500" transparent="1" foregroundColor="white" shadowColor="black" shadowOffset="-2,-2"/>
		<eLabel	position="10,50" size="1100,1" backgroundColor="grey"/>
		<widget source="timerlist" render="Listbox" position="10,60" size="1100,360" enableWrapAround="1" scrollbarMode="showOnDemand">
			<convert type="TemplatedMultiContent">
			{"template": [
			MultiContentEntryText(pos=(2,2), size=(1090,24), text = 3, font = 0, flags = RT_HALIGN_LEFT|RT_VALIGN_CENTER),
			MultiContentEntryText(pos=(2,26), size=(1090,30), text = 0, font = 1, flags = RT_HALIGN_LEFT|RT_VALIGN_CENTER),
			MultiContentEntryText(pos=(2,50), size=(400,20), text = 4, font = 1, flags = RT_HALIGN_LEFT|RT_VALIGN_CENTER),
			MultiContentEntryText(pos=(420,50), size=(670,20), text = 2, font = 1, flags = RT_HALIGN_RIGHT|RT_VALIGN_CENTER),
			],
			"fonts": [gFont("Regular",20),gFont("Regular",18)],
			"itemHeight": 72
			}
			</convert>
		</widget>
		<eLabel	position="10,430" size="1100,1" backgroundColor="grey"/>
		<widget font="Regular;20" position="10,440" render="Label" size="1100,30" source="Event">
			<convert type="EventName">Name</convert>
		</widget>
		<widget font="Regular;20" position="10,475" render="Label" size="1100,140" source="Event">
			<convert type="EventName">FullDescription</convert>
		</widget>
		</screen>"""

	def __init__(self, session, timers):
		Screen.__init__(self, session)

		# Sort timers by begin
		timers.sort(key = lambda x: x[1])
		self.sort_type = 0
		
		self.timers_org = timers

		showShortDescription = config.plugins.autotimer.preview_showShortDesc.value
		showBigPreView = config.plugins.autotimer.preview_showBigScreen.value
		
		if showBigPreView:
			self.skin = self.skinBig
			self.skinName = "AutoTimerPreviewBig"
		else:
			self.skin = self.skinNormal
			self.skinName = "AutoTimerPreview"
		
		self.epgcache = eEPGCache.getInstance()
		
		self.timers = []
		# timers = name, begin, end, serviceref, timername -> name, begin, timername, sname, timestr
		for x in timers:
			title = x[0]
			if showShortDescription:
				sref = ServiceReference(x[3])
				timepart  = x[1] + (x[2] - x[1]) / 4 # begin + (end - begin) / 4
				epgEvent = self.epgcache.lookupEventTime(sref.ref, timepart)
				if epgEvent and epgEvent.getEventName() == title and len(str(epgEvent.getShortDescription())):
					title += "  |  " + str(epgEvent.getShortDescription())
			
			# listvalues = name, begin, timername, sname, eventtimestr, sref, begin, end, org_name
			self.timers.append((title, x[1], x[4],ServiceReference(x[3]).getServiceName().replace('\xc2\x86', '').replace('\xc2\x87', '').encode('utf-8', 'ignore'),(("%s, %s ... %s (%d " + _("mins") + ")") % (FuzzyTime(x[1]) + FuzzyTime(x[2])[1:] + ((x[2] - x[1]) / 60,))), x[3], x[1], x[2], x[0]))

		self["timerlist"] = List(self.timers)
		
		self["Event"] = Event()
		self["timerlist"].onSelectionChanged.append(self.onSelectionChanged)

		# Initialize Buttons
		self["key_red"] = StaticText(_("Cancel"))
		self["key_yellow"] = StaticText()

		self.setSortDescription()

		# Define Actions
		self["actions"] = ActionMap(["SetupActions", "ColorActions", "InfobarEPGActions", "MenuActions"],
			{
				"cancel": self.cancel,
				"save": self.save,
				"yellow": self.sort,
				"showEventInfo": self.showEventInfo,
				"menu": self.menu
			}
		)

		self.onLayoutFinish.append(self.setCustomTitle)

	def setCustomTitle(self):
		self.setTitle(_("Preview AutoTimer"))
		self.onSelectionChanged()

	def setSortDescription(self):
		if self.sort_type == 1:
			self["key_yellow"].text = _("Sort Time")
		else:
			self["key_yellow"].text = _("Sort AutoTimer")

	def sort(self):
		timers = self.timers
		if timers:
			current = self["timerlist"].current
			idx = 0
			for timer in timers:
				if timer == current:
					break
				idx += 1
			if self.sort_type == 1:
				timers.sort(key=lambda x: x[1])
				self.sort_type = 0
			else:
				timers.sort(key = lambda x: x[4].lower())
				self.sort_type = 1
			self["timerlist"].updateList(timers)
			self["timerlist"].index = idx
			self.setSortDescription()

	def showEventInfo(self):
		current = self["timerlist"].current
		if current:
			sref = ServiceReference(current[5])
			begin = current[6]
			end   = current[7]
			part  = begin + (end - begin) / 4
			epgEvent = self.epgcache.lookupEventTime(sref.ref, part)
			
			from Screens.EventView import EventViewSimple, EventViewEPGSelect
			from Screens.InfoBar import InfoBar
			self.session.open(EventViewEPGSelect, epgEvent, sref, None, InfoBar.instance.openSingleServiceEPG, InfoBar.instance.openMultiServiceEPG, InfoBar.instance.openSimilarList)

	def menu(self):
		list = []
		if config.plugins.autotimer.preview_showShortDesc.value:
			list.append((_("hide shortdescription"), "hideshortdesc"))
		else:
			list.append((_("show shortdescription"), "showshortdesc"))
		
		if config.plugins.autotimer.preview_showBigScreen.value:
			list.append((_("show normal preview screen"), "shownormalpreview"))
		else:
			list.append((_("show big preview screen"), "showbigpreview"))

		self.session.openWithCallback(
			self.menuCallback,
			ChoiceBox,
			list = list,
		)

	def menuCallback(self, ret):
		ret = ret and ret[1]
		if ret:
			if ret == "shownormalpreview":
				self.hide()
				config.plugins.autotimer.preview_showBigScreen.value = False
				config.plugins.autotimer.preview_showBigScreen.save()
				self.session.open(AutoTimerPreview, self.timers_org)
				self.cancel()
				return
			
			elif ret == "showbigpreview":
				self.hide()
				config.plugins.autotimer.preview_showBigScreen.value = True
				config.plugins.autotimer.preview_showBigScreen.save()
				self.session.open(AutoTimerPreview, self.timers_org)
				self.cancel()
				return
			
			elif ret == "hideshortdesc":
				config.plugins.autotimer.preview_showShortDesc.value = False
			elif ret == "showshortdesc":
				config.plugins.autotimer.preview_showShortDesc.value = True
			config.plugins.autotimer.preview_showShortDesc.save()
			
			#add or remove shortdescription
			self.new_timers = []
			for x in self.timers:
				title = x[8] # org title
				if ret == "showshortdesc":
					sref = ServiceReference(x[5])
					timepart  = x[6] + (x[7] - x[6]) / 4 # begin + (end - begin) / 4
					epgEvent = self.epgcache.lookupEventTime(sref.ref, timepart)
					if epgEvent and len(str(epgEvent.getShortDescription())):
						title += "  |  " + str(epgEvent.getShortDescription())
				self.new_timers.append((title, x[1], x[2], x[3], x[4], x[5], x[6], x[7], x[8]))
			
			self.timers = self.new_timers
			del self.new_timers
			
			#refill timerlist
			self["timerlist"].updateList(self.timers)

	def onSelectionChanged(self):
		current = self["timerlist"].current
		if current:
			sref = ServiceReference(current[5])
			timepart  = current[6] + (current[7] - current[6]) / 4 # begin + (end - begin) / 4
			epgEvent = self.epgcache.lookupEventTime(sref.ref, timepart)
			self["Event"].newEvent(epgEvent)

	def cancel(self):
		self.close(None)

	def save(self):
		self.close(True)

